/* Procdures de Stevens renommes par RR
 *
 * Stevens       | RR
 * -----------------------------------
 * err_ret       | errno_return
 * err_sys       | errno_exit
 * err_dump      | errno_dump
 * err_msg       | err_return
 * err_quit      | err_exit
 * logno_msg     | logno_return
 * logno_sys     | logno_exit
 * log_msg       | log_return
 * log_sys       | log_exit
 * ------------------------------------
 */

#include	<errno.h>		/* for definition of errno */
#include	<stdarg.h>		/* ANSI C header file */
#include	<syslog.h>

#include	"stevens.h"

/* prdclaration des primitives prives */
static void	err_doit(int, const char *, va_list);
static void	log_doit(int, int, const char *, va_list ap);

/* Routines de traitement des erreurs */

void 
errno_return(const char *fmt, ...)
   /* Nonfatal error related to a system call.
    * Print a message and return. */
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	return;
}

void
errno_exit(const char *fmt, ...)
   /* Fatal error related to a system call.
    * Print a message and terminate. */
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	exit(1);
}

void
errno_dump(const char *fmt, ...)
   /* Fatal error related to a system call.
    * Print a message, dump core, and terminate. */
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	abort();		/* dump core and terminate */
	exit(1);		/* shouldn't get here */
}

void
err_return(const char *fmt, ...)
   /* Nonfatal error unrelated to a system call.
    * Print a message and return. */
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(0, fmt, ap);
	va_end(ap);
	return;
}

void
err_exit(const char *fmt, ...)
   /* Fatal error unrelated to a system call.
    * Print a message and terminate. */
{
	va_list		ap;

	va_start(ap, fmt);
	err_doit(0, fmt, ap);
	va_end(ap);
	exit(1);
}

/* --------- errorlog.c ---------------- */

/* Attention : modifier les routines qui appelent log_open en leur ajoutant
 * le paramtre debug  0
 */

/* Error routines for programs that can run as a daemon. */

static int my_debug;

void
log_open(int debug, const char *ident, int option, int facility)
	  /* Initialize syslog(), if running as daemon.
   	* if debug is nonzero set, it's interactive, else it's a daemon 
		*/
{  
  my_debug = debug;
	if (debug == 0)
		openlog(ident, option, facility);
}

void
logno_return(const char *fmt, ...)
	  /* Nonfatal error related to a system call.
		* Print a message with the system's errno value and return. */
{
	va_list		ap;

	va_start(ap, fmt);
	log_doit(1, LOG_ERR, fmt, ap);
	va_end(ap);
	return;
}

void
logno_exit(const char *fmt, ...)
	  /* Fatal error related to a system call.
		* Print a message with the system's errno value and terminate. */
{
	va_list		ap;

	va_start(ap, fmt);
	log_doit(1, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(2);
}

void
log_return(const char *fmt, ...)
	  /* Nonfatal error unrelated to a system call.
		* Print a message and return. */
{
	va_list		ap;

	va_start(ap, fmt);
	log_doit(0, LOG_ERR, fmt, ap);
	va_end(ap);
	return;
}

void
log_exit(const char *fmt, ...)
	  /* Fatal error unrelated to a system call.
		* Print a message and terminate. */
{
	va_list		ap;

	va_start(ap, fmt);
	log_doit(0, LOG_ERR, fmt, ap);
	va_end(ap);
	exit(2);
}

/* Private part */

static void
err_doit(int errnoflag, const char *fmt, va_list ap)
	  /* Print a message and return to caller.
		* Caller specifies "errnoflag". */
{
	int		errno_save;
	char	buf[MAXLINE];

	errno_save = errno;		/* value caller might want printed */
	vsprintf(buf, fmt, ap);
	if (errnoflag)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	strcat(buf, "\n");
	fflush(stdout);		/* in case stdout and stderr are the same */
	fputs(buf, stderr);
	fflush(stderr);		/* SunOS 4.1.* doesn't grok NULL argument */
	return;
}

static void
log_doit(int errnoflag, int priority, const char *fmt, va_list ap)
	  /* Print a message and return to caller.
		* Caller specifies "errnoflag" and "priority". */
{
	int		errno_save;
	char	buf[MAXLINE];

	errno_save = errno;		/* value caller might want printed */
	vsprintf(buf, fmt, ap);
	if (errnoflag)
		sprintf(buf+strlen(buf), ": %s", strerror(errno_save));
	strcat(buf, "\n");
	if (my_debug) {
		fflush(stdout);
		fputs(buf, stderr);
		fflush(stderr);
	} else
		syslog(priority, buf);
	return;
}
